function [OutCellArray, Outconfig]=PSD(InCellArray, Inconfig)
%PSD calculates a 2D periodogram of a map
%
% The PSD is calculated as:
%
%       PSD = (fft.*conj(fft)) / area
%
% where "area" is defined as (xSize * ySize) of the input map. 
%
% The "fft" variable in the PSD calculation is determined using the MATLAB
% fft command multiplied by the pixel spacing. The multiplication is used
% to maintain units that arise during integration. Note that the MATLAB fft
% command uses a normalization of 1. This is why the division by area is
% needed in the PSD calculation.
%
% Inputs: Map
% Outputs: Frequency Map
%
% Author: Ping Zhou
% Email:
%
% Version history:
% 9/9/10 - Version 1
% 6/21/12 - Version 2 - Greg Smith - normalized output to keep units simple
% 10/10/2012 - Version 2.1 - Greg Smith - changed output units to better match expectations

OutCellArray{1}=InCellArray{1};
OutCellArray{1}.type='freqmap';

xsize=str2double(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'X Size'),2});
ysize=str2double(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Y Size'),2});

xunit=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'X Unit'),2};
yunit=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Y Unit'),2};
zunit=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Z Unit'),2};

% handle output for different combinations of x,y,z units
if strcmpi(xunit,yunit) && strcmpi(xunit,zunit), zunit = [zunit,'^4'];
elseif strcmpi(xunit,yunit), zunit = [zunit,'^2 ',xunit,'^2'];
elseif strcmpi(xunit,zunit), zunit = [zunit,'^3 ',yunit];
elseif strcmpi(yunit,zunit), zunit = [zunit,'^3 ',xunit];
else zunit = [zunit,'^2 ',xunit,' ',yunit];
end

% generate header information
OutCellArray{1}.header = [];
OutCellArray{1}.header={'Date Created', ''};
OutCellArray{1}.header(end+1,:)={'X unit', ['1/', xunit]};
OutCellArray{1}.header(end+1,:)={'Y unit', ['1/', yunit]};
OutCellArray{1}.header(end+1,:)={'Z unit', zunit};

delx = xsize/size(InCellArray{1}.data, 2);          % size of each pixel in x-direction
xsize = 1/(2*delx);                                 % max frequency in x direction

OutCellArray{1}.header(end+1,:)={'X Max Frequency', num2str(xsize)};

dely = ysize/size(InCellArray{1}.data, 1);
ysize = 1/(2*dely);

OutCellArray{1}.header(end+1,:)={'Y Max Frequency', num2str(ysize)};

Outconfig=Inconfig;

data = InCellArray{1}.data;

data(isnan(data))=0;

G = delx*dely*fftshift(fft2(fftshift(data)));
G = G.*conj(G);

% G.S. - 2012/06/21 - divide by square of area of map to keep output units simple
% G.S. - 2012/10/10 - remove extra division - although units are more complicated, they better represent typical usage
xsize=str2double(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'X Size'),2});
ysize=str2double(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Y Size'),2});
OutCellArray{1}.data=G/xsize/ysize;

