function [OutCellArray, OutConfig] = KeepDatatypeSame(InCellArray, InConfig)
% KeepDatatypeSame is a template module that outputs same datatype as given.
%
% KeepDatatypeSame does not perform any action. It is intended to show an
% example of a module which can operate on many data types. The output of
% this template module will be the same data type as the input. If a 'map'
% data type is input, then a 'map' data type will be output.
%
% This module is intended to demonstrate how modules can be created to work
% on multiple map types, but in the case where the output map is the same
% as the input map. This module could be greatly simplified, but the steps
% have been expanded to show how they might be used in a typical
% application.
%
% Note: This module will NOT APPEAR IN SAGUARO. To practice using it, place
% a copy of the .m file and the associated .txt file in the 'UserModules'
% directory.
%
% Note: The associated KeepDatatypeSame.txt file contains the following lines:
%                   Coordinates, 1, General
%                   FreqMap, 1, General
%                   General, 1, General
%                   LayerMap, 1, General
%                   Map, 1, General
%                   Mask, 1, General
%                   Zernike, 1, General
% The first value on each line describes the input data type. In this case,
% the module will be available for any of the defined SAGUARO data types
% (Coordinates, FreqMap, General, LayerMap, Map, Mask, Zernike).
%
% The second value on each line describes the number of input data sets
% required. In this case, each data type requires 1 input.
%
% The third value in each line in the text file describes the category the
% module fits into. In this case, the module can be found under the
% 'General' category. This categorization is useful for quickly finding
% modules in SAGUARO by using the sort feature.
%
% Additional lines in the text file would denote module configuration
% parameters. These lines start with a '%' symbol. This module requires no
% configuration parameters.
%
%
% Inputs: One module of any type
% Outputs: Same data as input
%
% Author: Greg Smith
% Email: gsmith@optics.arizona.edu
%
% Version history:
% 2012/06/13 - Version 1.0

    %% Set default values
    % If the calculation reaches an early 'return' statment, no output will be returned.
    % In this simple module, the variables are actually unused and generate
    % an mLint warning which must be suppressed. The code is included here
    % to demonstrate which values are best to return when a user 'aborts'
    % the module computation. In normal computation, it is better to
    % eliminate the orange mLint warnings on the right side of the command
    % window than to suppress them with use of the special comment '%#ok'.
    OutConfig = [];         %#ok - G.S. June 13, 2012 - mLint warning about unused variable can be suppressed for the purposes of this demonstration.
    OutCellArray = [];      %#ok - G.S. June 13, 2012 - mLint warning about unused variable can be suppressed for the purposes of this demonstration.
    
    
    %% update OutCellArray and OutConfig values based on SAGUARO data type
    % Because this module uses no configuration parameters, the OutConfig
    % will always be the same as the InConfig.
    OutConfig = InConfig;
    
    % Get the input data from InCellArray
    % Because there is only 1 input, the data is always in the first part
    % of the InCellArray structure.
    currentData = InCellArray{1}.data;

    % Below would be the place where you could put processing code that
    % extracts relevant InCellArray values, performs a computation and
    % places them into the corresponding OutCellArray values. For this
    % template, only the data changes and the header information stays the
    % same, so the InCellArray is copied over to the OutCellArray and the
    % data is updated.
    switch lower(InCellArray{1}.type)
        case 'coordinates'
            OutCellArray = InCellArray;
            OutCellArray{1}.data = currentData;
        case 'freqmap'
            OutCellArray = InCellArray;
            OutCellArray{1}.data = currentData;
        case 'general'
            OutCellArray = InCellArray;
            OutCellArray{1}.data = currentData;
        case 'layermap'
            OutCellArray = InCellArray;
            OutCellArray{1}.data = currentData;
        case 'map'
            OutCellArray = InCellArray;
            OutCellArray{1}.data = currentData;
        case 'mask'
            OutCellArray = InCellArray;
            OutCellArray{1}.data = currentData;
        case 'zernike'
            OutCellArray = InCellArray;
            OutCellArray{1}.data = currentData;
        otherwise
            % unsupported data type detected - warn user and return empty data
            warndlg('Unsupported data type detected! Computation will be cancelled',...
                'KeepDatatypeSame - Unsupported Data Type',...
                'modal');
            OutCellArray = [];
    end
end